
;; superseded-module.scm
;; Compatibility module for deprecated (gnucash superseded-module).
;; This file is autogenerated, do not modify by hand.

(define-module (gnucash superseded-module))

(issue-deprecation-warning "* WARN <gnc-guile-deprecation> *: Module '(gnucash superseded-module)' has been deprecated and will be removed in the future.")
                (issue-deprecation-warning "* WARN <gnc-guile-deprecation> *: Use module '(gnucash utilities)' instead.")

(use-modules (gnucash utilities))

(let ((i (module-public-interface (current-module))))
     (module-use! i (resolve-interface '(gnucash utilities))))