
;; deprecated-module.scm
;; Compatibility module for deprecated (gnucash deprecated-module).
;; This file is autogenerated, do not modify by hand.

(define-module (gnucash deprecated-module))

(issue-deprecation-warning "* WARN <gnc-guile-deprecation> *: Module '(gnucash deprecated-module)' has been deprecated and will be removed in the future.")
